@@ -1,6 +1,5 @@
-#include <gtest/gtest.h>
+#include <pcl/test/gtest.h>
 #include <pcl/point_types.h>
-#include <pcl/common/io.h>
 #include <pcl/io/pcd_io.h>
 #include <pcl/io/pcd_grabber.h>
 #include <pcl/io/image_grabber.h>
@@ -8,16 +7,17 @@
 #include <string>
 #include <thread>
 #include <vector>
+#include <boost/filesystem.hpp> // for directory_iterator, extension
+#include <boost/algorithm/string/case_conv.hpp> // for to_upper_copy
 
-using namespace std;
 using namespace std::chrono_literals;
 
 using PointT = pcl::PointXYZRGBA;
 using CloudT = pcl::PointCloud<PointT>;
 
-string tiff_dir_;
-string pclzf_dir_;
-string pcd_dir_;
+std::string tiff_dir_;
+std::string pclzf_dir_;
+std::string pcd_dir_;
 std::vector<CloudT::ConstPtr> pcds_;
 std::vector<std::string> pcd_files_;
 
